/* CSS decorations for simple api */

body {
    margin: 5px 20px;
    padding: 0;
    background-color: #ffffff;
    font-family: "Helvetica Neue", Helvetica, Arial, sans-serif;
}

h1 {
    text-align: center;
}

br {
    display: none;
}

a:nth-child(even) {
    background-color: #ededed;
}

a:nth-child(odd) {
    background-color: #f9f9f9;
}

a {
    margin: 3px;
    padding: 5px 10px;
    display: block;
    border: 1px solid #f9f9f9;
    border-radius: 4px;
    color: #204d74;
    text-decoration: none;
}

a.package:before {
    content: "Package: ";
}

a.file:before {
    content: "File: ";
}